package builder

import (
	"institutionMS/external"
	"institutionMS/layers/api"
	"institutionMS/layers/service"
	"institutionMS/middleware"
	"os"
)

// CreateAuthRouter creates authentication router
func CreateAuthRouter() (router api.Api) {
	ext := createAuthExternalApi()
	svc := createAuthService(ext)
	mid := createAuthMiddleware()
	router = createAuthApi(svc, mid)
	return
}

// createExternalApi creates auth external API
func createAuthExternalApi() (ext external.AuthAPI){
	ext = external.AuthAPI{
		Domain:     os.Getenv("AUTH_DOMAIN"),
		MgmtToken:  os.Getenv("AUTH_MGMT_TOKEN"),
		Connection: os.Getenv("AUTH_CONNECTION"),
		ClientId:   os.Getenv("AUTH_CLIENT_ID"),
		Secret:     os.Getenv("AUTH_CLIENT_SECRET"),
	}
	return
}

// createAuthService creates service for authentication
func createAuthService(ext external.AuthAPI) (svc service.Service) {
	svc = service.AuthService{
		Api: ext,
	}
	return
}

// createAuthMiddleware creates authorization logic
func createAuthMiddleware() (mid middleware.Middleware) {
	mid = &middleware.AuthMiddleware{
		Secret:   os.Getenv("AUTH_CLIENT_SECRET"),
		ClientId: os.Getenv("AUTH_CLIENT_ID"),
		Domain:   os.Getenv("AUTH_DOMAIN"),
	}
	mid.New()
	return
}

// createAuthApi creates authentication API
func createAuthApi(svc service.Service, mid middleware.Middleware) (router api.Api) {
	router = api.AuthAPI {
		Middleware: mid,
		Service: svc,
	}
	return
}
