/*
	-> Este é um arquivo base para desenvolvimento para a criação da dependência de camadas
		da aplicação, utilizada no pacote principal e de testes;
	-> Codifique em inglês utilizando os termos definidos no glossário;
	-> Qualquer dúvida contate um AGES 3.
*/
package builder

import (
	"institutionMS/external"
	"institutionMS/layers/api"
	"institutionMS/layers/infrastructure"
	"institutionMS/layers/repository"
	"institutionMS/layers/service"
	"institutionMS/middleware"
	"os"
)

// CreateInstitutionRouter creates application router through layer dependency chain
func CreateInstitutionRouter() (router api.Api) {
	db := createInfrastructure()
	repo := createInstitutionRepository(db)
	iSvc := createInstitutionService(repo)
	ext := createOpportunityExternalApi()
	oSvc := createOpportunityService(ext)
	mid := createInstitutionMiddleware()
	router = createInstitutionApi(iSvc, oSvc, mid)
	return
}

// createInfrastructure creates an database for repository layer
func createInfrastructure() (db infrastructure.DocumentDatabase) {
	db = func() *infrastructure.MongoDB {
		return &infrastructure.MongoDB{
			Username: os.Getenv("MONGO_USER"),
			Password: os.Getenv("MONGO_PASSWORD"),
			Host:     os.Getenv("MONGO_HOST"),
			Port:     os.Getenv("MONGO_PORT"),
		}
	}()
	return
}

// createInstitutionRepository creates a repository for service layer
func createInstitutionRepository(db infrastructure.DocumentDatabase) (repo repository.Repository) {
	repo = repository.InstitutionRepository{
		Database:                 db,
		TargetDocumentDatabase:   os.Getenv("MONGO_DATABASE"),
		TargetDocumentCollection: os.Getenv("MONGO_COLLECTION_NAME"),
	}
	return
}

// createInstitutionService creates a service for api layer
func createInstitutionService(repo repository.Repository) (svc service.Service) {
	svc = service.InstitutionService{
		Repository: repo,
	}
	return
}

func createOpportunityExternalApi() (api external.OpportunityAPI) {
	api = external.OpportunityAPI{
		Address: os.Getenv("OPPORTUNITY_ADDRESS"),
	}
	return
}

func createOpportunityService(ext external.OpportunityAPI) (svc service.Service) {
	svc = service.OpportunityService{
		Api: ext,
	}
	return
}

// createInstitutionMiddleware creates authorization logic
func createInstitutionMiddleware() (mid middleware.Middleware) {
	mid = &middleware.AuthMiddleware{
		Secret:   os.Getenv("AUTH_CLIENT_SECRET"),
		ClientId: os.Getenv("AUTH_CLIENT_ID"),
		Domain:   os.Getenv("AUTH_DOMAIN"),
	}
	mid.New()
	return
}


// createInstitutionApi creates an api router
func createInstitutionApi(iSvc service.Service, oSvc service.Service, mid middleware.Middleware) (router api.Api) {
	router = api.InstitutionApi{
		Middleware:			mid,
		InstitutionService:	iSvc,
		OpportunityService: oSvc,
	}
	return
}

