package api

import (
	"github.com/gin-gonic/gin"
	"institutionMS/layers/service"
	"institutionMS/middleware"
)

// AuthAPI is a struct that defines authentication at api layer
type AuthAPI struct {
	Middleware middleware.Middleware
	Service    service.Service
}

// CreateRouter creates this api router for gin to consume
func (api AuthAPI) CreateRouter(router *gin.Engine) {
	api.createInstitutionLogin(router)
	api.createInstitutionSignup(router)
	api.createInstitutionAuthMgmt(router)
}

// createInstitutionSignup creates Institution signup route for gin to consume
func (api AuthAPI) createInstitutionSignup(router *gin.Engine) {
	logInRouter := router.Group("/institution/auth/signup")
	{
		logInRouter.POST("", api.Service.Create)
	}
}

// createInstitutionLogin creates Institution login route for gin to consume
func (api AuthAPI) createInstitutionLogin(router *gin.Engine) {
	signInRouter := router.Group("/institution/auth/login")
	{
		signInRouter.POST("", api.Service.Read)
	}
}

// createInstitutionAuthMgmt creates Institution management routes for gin to consume
func (api AuthAPI) createInstitutionAuthMgmt(router *gin.Engine) {
	mgmtRouter := router.Group("/institution/auth/mgmt")
	{
		mgmtRouter.Use(api.Middleware.MiddlewareFunction())
		mgmtRouter.PUT("", api.Service.Update)
		mgmtRouter.DELETE("", api.Service.Delete)
	}
}
