package api

import (
	"institutionMS/layers/service"
	"github.com/gin-gonic/gin"
	"institutionMS/middleware"
)

// InstitutionApi is a struct that defines api layer behaviour for institution
type InstitutionApi struct {
	Middleware middleware.Middleware
	InstitutionService service.Service
	OpportunityService service.Service
}

// CreateRouter creates Institution's Api routers for gin to consume
func (api InstitutionApi) CreateRouter(router *gin.Engine) {
	api.createInstitutionProfileRouter(router)
	api.createOpportunityMgmtRouter(router)
}


// createInstitutionProfileRouter creates a Institution's profile Api router for gin to consume
func (api InstitutionApi) createInstitutionProfileRouter(router *gin.Engine) {
	profileRouter := router.Group("/institutions")
	{
		profileRouter.Use(api.Middleware.MiddlewareFunction())
		profileRouter.POST("", api.InstitutionService.Create)
		profileRouter.GET("", api.InstitutionService.Read)
		profileRouter.PUT(":id", api.InstitutionService.Update)
		profileRouter.DELETE(":id", api.InstitutionService.Delete)
	}
}

func(api InstitutionApi) createOpportunityMgmtRouter(router *gin.Engine) {
	opportunityRouter := router.Group("/institution/opportunity")
	{
		opportunityRouter.Use(api.Middleware.MiddlewareFunction())
		opportunityRouter.POST("", api.OpportunityService.Create)
		opportunityRouter.GET(":id", api.OpportunityService.Read)
		opportunityRouter.PUT(":id", api.OpportunityService.Update)
		opportunityRouter.DELETE(":id", api.OpportunityService.Delete)
	}
}
