package repository

import (
	"go.mongodb.org/mongo-driver/bson/primitive"
	"institutionMS/layers/entity"
	"institutionMS/layers/infrastructure"
)

// InstitutionRepository is a struct that defines a institution Repository behaviour at repository layer
type InstitutionRepository struct {
	Database                 infrastructure.DocumentDatabase
	TargetDocumentDatabase   string
	TargetDocumentCollection string
}

// Create inserts a Institution in TargetDocumentDatabase and returns its given id
func (repository InstitutionRepository) Create(institution entity.Institution) (id string, err error) {
	repository.Database.Connect()
	defer repository.Database.Disconnect()
	repository.Database.Collection(repository.TargetDocumentDatabase, repository.TargetDocumentCollection)
	id, err = repository.Database.Create(institution)
	return
}

// Read searches for a Institution in a TargetDocumentDatabase based on email
func (repository InstitutionRepository) Read(email string) (institution *entity.Institution, err error) {
	repository.Database.Connect()
	defer repository.Database.Disconnect()
	repository.Database.Collection(repository.TargetDocumentDatabase, repository.TargetDocumentCollection)
	institution, err = repository.Database.Read(email)
	return
}

// Update a Institution in a TargetDocumentDatabase based on id
func (repository InstitutionRepository) Update(objId primitive.ObjectID, institution *entity.Institution) (updated *entity.Institution, err error) {
	repository.Database.Connect()
	defer repository.Database.Disconnect()
	repository.Database.Collection(repository.TargetDocumentDatabase, repository.TargetDocumentCollection)
	return repository.Database.Update(objId, institution)
}

// Delete a Institution in TargetDocumentDatabase
func (repository InstitutionRepository) Delete(objId primitive.ObjectID) (deleted *entity.Institution, err error) {
	repository.Database.Connect()
	defer repository.Database.Disconnect()
	repository.Database.Collection(repository.TargetDocumentDatabase, repository.TargetDocumentCollection)
	return repository.Database.Delete(objId)
}
