package middleware

import (
	"encoding/base64"
	"fmt"
	"github.com/auth0-community/go-auth0"
	"github.com/gin-gonic/gin"
	"gopkg.in/square/go-jose.v2"
)

// AuthMiddleware is a struct that defines authorization middleware for this API
type AuthMiddleware struct {
	Secret string
	ClientId string
	Domain string
	validator *auth0.JWTValidator
}

// New is a constructor for AuthMiddleware
func (mid *AuthMiddleware) New() {
	secret, _ := base64.URLEncoding.DecodeString(mid.Secret)
	audience := []string{mid.ClientId}
	issuer := fmt.Sprintf("https://%s/", mid.Domain)
	secretProvider := auth0.NewKeyProvider(secret)
	configuration := auth0.NewConfiguration(secretProvider, audience, issuer, jose.HS256)
	mid.validator = auth0.NewValidator(configuration, nil)
}

// MiddlewareFunction is a middleware for authorization
func (mid AuthMiddleware) MiddlewareFunction() gin.HandlerFunc{
	return func (ctx *gin.Context) {
		_, err := mid.validator.ValidateRequest(ctx.Request)
		if err != nil {
			ctx.Set("authorized", false)
		} else {
			ctx.Set("authorized", true)
		}
		ctx.Next()
	}
}
