package main

import (
	"institutionMS/builder"
	"encoding/base64"
	"fmt"
	"github.com/auth0-community/go-auth0"
	"github.com/gin-gonic/gin"
	"github.com/joho/godotenv"
	"gopkg.in/square/go-jose.v2"
	"log"
	"os"

)

const env = ".env"

func main() {
	err := godotenv.Load(env)

	if err != nil {
		log.Fatal("Error loading .env file")
	}

	ginRouter := gin.Default()
	ginRouter.Use(Validate())
	appRouter := builder.CreateInstitutionRouter()
	appRouter.CreateRouter(ginRouter)
	authRouter := builder.CreateAuthRouter()
	authRouter.CreateRouter(ginRouter)
	err = ginRouter.Run("0.0.0.0:8080")

	if err != nil {
		log.Fatal(error.Error(err))
	}
}
func Validate() gin.HandlerFunc{
	return func (ctx *gin.Context) {
		secret, _ := base64.URLEncoding.DecodeString(os.Getenv("AUTH_CLIENT_SECRET"))
		audience := []string{os.Getenv("AUTH_CLIENT_ID")}
		issuer := fmt.Sprintf("https://%s/", os.Getenv("AUTH_DOMAIN"))

		secretProvider := auth0.NewKeyProvider(secret)
		configuration := auth0.NewConfiguration(secretProvider, audience, issuer, jose.HS256)
		validator := auth0.NewValidator(configuration, nil)

		token, err := validator.ValidateRequest(ctx.Request)

		if err != nil {
			fmt.Println(token)
			fmt.Println(err)
			ctx.Set("authorized", false)
		} else {
			ctx.Set("authorized", true)
		}
		ctx.Next()
	}
}

