package test

import (
	"bytes"
	"encoding/json"
	"github.com/gin-gonic/gin"
	"github.com/joho/godotenv"
	"github.com/stretchr/testify/assert"
	"institutionMS/builder"
	"io/ioutil"
	"log"
	"net/http"
	"net/http/httptest"
	"os"
	"testing"
)

var token string
var authBody = `{"email":"testuser@email.com","password":"pass1234@A"}`

// setToken extracts 'token' field from POST method to be available for other tests
func setToken(w *httptest.ResponseRecorder) {
	b, err := ioutil.ReadAll(w.Body)
	var data map[string]interface{}
	if err != nil {
		log.Fatal(err)
	}
	err = json.Unmarshal(b, &data)
	if err != nil {
		log.Fatal(err)
	}
	token = data["token"].(string)
}

// setupAuthRouter is a private function that loads Gin router
func setupAuthRouter() (ginRouter *gin.Engine) {
	err := godotenv.Load("../.env")

	if err != nil {
		log.Fatal("Error loading .env file")
	}

	os.Setenv("NO_AUTH", "false")

	ginRouter = gin.Default()
	appRouter := builder.CreateAuthRouter()
	appRouter.CreateRouter(ginRouter)
	return
}

// TestSuccessAuthSignup tests Auth POST signup method with success response
func TestSuccessAuthSignup(t *testing.T) {
	router := setupAuthRouter()
	json := []byte(authBody)

	w := httptest.NewRecorder()
	req, _ := http.NewRequest("POST", "/institution/auth/signup", bytes.NewBuffer(json))
	req.Header.Add("Content-Type", "application/json")
	router.ServeHTTP(w, req)

	assert.Equal(t, http.StatusCreated, w.Code)
}

// TestSuccessAuthLogin tests Auth POST login method with success response
func TestSuccessAuthLogin(t *testing.T) {
	router := setupAuthRouter()
	json := []byte(authBody)

	w := httptest.NewRecorder()
	req, _ := http.NewRequest("POST", "/institution/auth/login", bytes.NewBuffer(json))
	req.Header.Add("Content-Type", "application/json")
	router.ServeHTTP(w, req)

	setToken(w)

	assert.Equal(t, http.StatusOK, w.Code)
}

// TestSuccessAuthDelete tests Auth DELETE method with success response
func TestSuccessAuthDelete(t *testing.T) {
	router := setupAuthRouter()

	w := httptest.NewRecorder()
	req, _ := http.NewRequest("DELETE", "/institution/auth/mgmt?email=testuser@email.com", nil)
	req.Header.Add("Content-Type", "application/json")
	req.Header.Add("Authorization", "Bearer "+token)
	router.ServeHTTP(w, req)

	assert.Equal(t, http.StatusOK, w.Code)
}
